@@ -16,13 +16,15 @@
 """
 Key manager implementation for Barbican
 """
+import calendar
 import time
 
 from cryptography.hazmat import backends
 from cryptography.hazmat.primitives import serialization
 from cryptography import x509 as cryptography_x509
-from keystoneclient.auth import identity
-from keystoneclient import session
+from keystoneauth1 import identity
+from keystoneauth1 import loading
+from keystoneauth1 import session
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import excutils
@@ -36,12 +38,14 @@ from castellan.common.objects import public_key as pub_key
 from castellan.common.objects import symmetric_key as sym_key
 from castellan.common.objects import x_509
 from castellan.key_manager import key_manager
-from castellan.openstack.common import _i18n as u
+from castellan.i18n import _, _LE, _LI
 
 from barbicanclient import client as barbican_client
 from barbicanclient import exceptions as barbican_exceptions
+from oslo_utils import timeutils
 from six.moves import urllib
 
+
 barbican_opts = [
     cfg.StrOpt('barbican_endpoint',
                help='Use this endpoint to connect to Barbican, for example: '
@@ -82,7 +86,7 @@ class BarbicanKeyManager(key_manager.KeyManager):
         self._base_url = None
         self.conf = configuration
         self.conf.register_opts(barbican_opts, group=BARBICAN_OPT_GROUP)
-        session.Session.register_conf_options(self.conf, BARBICAN_OPT_GROUP)
+        loading.register_session_conf_options(self.conf, BARBICAN_OPT_GROUP)
 
     def _get_barbican_client(self, context):
         """Creates a client to connect to the Barbican service.
@@ -96,21 +100,14 @@ class BarbicanKeyManager(key_manager.KeyManager):
 
         # Confirm context is provided, if not raise forbidden
         if not context:
-            msg = u._("User is not authorized to use key manager.")
+            msg = _LE("User is not authorized to use key manager.")
             LOG.error(msg)
             raise exception.Forbidden(msg)
 
-        if not hasattr(context, 'tenant') or context.tenant is None:
-            msg = u._("Unable to create Barbican Client without tenant "
-                      "attribute in context object.")
-            LOG.error(msg)
-            raise exception.KeyManagerError(reason=msg)
-
         if self._barbican_client and self._current_context == context:
             return self._barbican_client
 
         try:
-            self._current_context = context
             auth = self._get_keystone_auth(context)
             sess = session.Session(auth=auth)
 
@@ -118,9 +115,10 @@ class BarbicanKeyManager(key_manager.KeyManager):
             self._barbican_client = barbican_client.Client(
                 session=sess,
                 endpoint=self._barbican_endpoint)
+            self._current_context = context
 
         except Exception as e:
-            LOG.error(u._LE("Error creating Barbican client: %s"), e)
+            LOG.error(_LE("Error creating Barbican client: %s"), e)
             raise exception.KeyManagerError(reason=e)
 
         self._base_url = self._create_base_url(auth,
@@ -130,14 +128,48 @@ class BarbicanKeyManager(key_manager.KeyManager):
         return self._barbican_client
 
     def _get_keystone_auth(self, context):
-        # TODO(kfarr): support keystone v2
-        auth = identity.v3.Token(
-            auth_url=self.conf.barbican.auth_endpoint,
-            token=context.auth_token,
-            project_id=context.tenant,
-            domain_id=context.user_domain,
-            project_domain_id=context.project_domain)
-        return auth
+        auth_url = self.conf.barbican.auth_endpoint
+
+        if context.__class__.__name__ is 'KeystonePassword':
+            return identity.Password(
+                auth_url=auth_url,
+                username=context.username,
+                password=context.password,
+                user_id=context.user_id,
+                user_domain_id=context.user_domain_id,
+                user_domain_name=context.user_domain_name,
+                trust_id=context.trust_id,
+                domain_id=context.domain_id,
+                domain_name=context.domain_name,
+                project_id=context.project_id,
+                project_name=context.project_name,
+                project_domain_id=context.project_domain_id,
+                project_domain_name=context.project_domain_name,
+                reauthenticate=context.reauthenticate)
+        elif context.__class__.__name__ is 'KeystoneToken':
+            return identity.Token(
+                auth_url=auth_url,
+                token=context.token,
+                trust_id=context.trust_id,
+                domain_id=context.domain_id,
+                domain_name=context.domain_name,
+                project_id=context.project_id,
+                project_name=context.project_name,
+                project_domain_id=context.project_domain_id,
+                project_domain_name=context.project_domain_name,
+                reauthenticate=context.reauthenticate)
+        # this will be kept for oslo.context compatibility until
+        # projects begin to use utils.credential_factory
+        elif context.__class__.__name__ is 'RequestContext':
+            return identity.Token(
+                auth_url=auth_url,
+                token=context.auth_token,
+                project_id=context.tenant)
+        else:
+            msg = _LE("context must be of type KeystonePassword, "
+                      "KeystoneToken, or RequestContext.")
+            LOG.error(msg)
+            raise exception.Forbidden(reason=msg)
 
     def _get_barbican_endpoint(self, auth, sess):
         if self.conf.barbican.barbican_endpoint:
@@ -155,7 +187,7 @@ class BarbicanKeyManager(key_manager.KeyManager):
             discovery = auth.get_discovery(sess, url=endpoint)
             raw_data = discovery.raw_version_data()
             if len(raw_data) == 0:
-                msg = u._LE(
+                msg = _LE(
                     "Could not find discovery information for %s") % endpoint
                 LOG.error(msg)
                 raise exception.KeyManagerError(reason=msg)
@@ -193,7 +225,7 @@ class BarbicanKeyManager(key_manager.KeyManager):
         except (barbican_exceptions.HTTPAuthError,
                 barbican_exceptions.HTTPClientError,
                 barbican_exceptions.HTTPServerError) as e:
-            LOG.error(u._LE("Error creating key: %s"), e)
+            LOG.error(_LE("Error creating key: %s"), e)
             raise exception.KeyManagerError(reason=e)
 
     def create_key_pair(self, context, algorithm, length,
@@ -231,7 +263,7 @@ class BarbicanKeyManager(key_manager.KeyManager):
         except (barbican_exceptions.HTTPAuthError,
                 barbican_exceptions.HTTPClientError,
                 barbican_exceptions.HTTPServerError) as e:
-            LOG.error(u._LE("Error creating key pair: %s"), e)
+            LOG.error(_LE("Error creating key pair: %s"), e)
             raise exception.KeyManagerError(reason=e)
 
     def _get_barbican_object(self, barbican_client, managed_object):
@@ -310,7 +342,7 @@ class BarbicanKeyManager(key_manager.KeyManager):
         except (barbican_exceptions.HTTPAuthError,
                 barbican_exceptions.HTTPClientError,
                 barbican_exceptions.HTTPServerError) as e:
-            LOG.error(u._LE("Error storing object: %s"), e)
+            LOG.error(_LE("Error storing object: %s"), e)
             raise exception.KeyManagerError(reason=e)
 
     def _create_secret_ref(self, object_id):
@@ -320,7 +352,7 @@ class BarbicanKeyManager(key_manager.KeyManager):
         :return: the URL of the requested secret
         """
         if not object_id:
-            msg = "Key ID is None"
+            msg = _("Key ID is None")
             raise exception.KeyManagerError(reason=msg)
         base_url = self._base_url
         if base_url[-1] != '/':
@@ -333,31 +365,40 @@ class BarbicanKeyManager(key_manager.KeyManager):
         Barbican key creation is done asynchronously, so this loop continues
         checking until the order is active or a timeout occurs.
         """
-        active = u'ACTIVE'
+        active_status = u'ACTIVE'
+        error_status = u'ERROR'
         number_of_retries = self.conf.barbican.number_of_retries
         retry_delay = self.conf.barbican.retry_delay
         order = barbican_client.orders.get(order_ref)
         time.sleep(.25)
         for n in range(number_of_retries):
-            if order.status != active:
+            if order.status == error_status:
+                kwargs = {"status": error_status,
+                          "code": order.error_status_code,
+                          "reason": order.error_reason}
+                msg = _LE("Order is in %(status)s status - status code: "
+                          "%(code)s, status reason: %(reason)s") % kwargs
+                LOG.error(msg)
+                raise exception.KeyManagerError(reason=msg)
+            if order.status != active_status:
                 kwargs = {'attempt': n,
                           'total': number_of_retries,
                           'status': order.status,
-                          'active': active,
+                          'active': active_status,
                           'delay': retry_delay}
-                msg = u._LI("Retry attempt #%(attempt)i out of %(total)i: "
-                            "Order status is '%(status)s'. Waiting for "
-                            "'%(active)s', will retry in %(delay)s "
-                            "seconds")
+                msg = _LI("Retry attempt #%(attempt)i out of %(total)i: "
+                          "Order status is '%(status)s'. Waiting for "
+                          "'%(active)s', will retry in %(delay)s "
+                          "seconds")
                 LOG.info(msg, kwargs)
                 time.sleep(retry_delay)
                 order = barbican_client.orders.get(order_ref)
             else:
                 return order
-        msg = u._LE("Exceeded retries: Failed to find '%(active)s' status "
-                    "within %(num_retries)i retries") % {'active': active,
-                                                         'num_retries':
-                                                         number_of_retries}
+        msg = _LE("Exceeded retries: Failed to find '%(active)s' status "
+                  "within %(num_retries)i retries") % {
+            'active': active_status,
+            'num_retries': number_of_retries}
         LOG.error(msg)
         raise exception.KeyManagerError(reason=msg)
 
@@ -421,14 +462,22 @@ class BarbicanKeyManager(key_manager.KeyManager):
 
         secret_data = self._get_secret_data(secret)
 
+        # convert created ISO8601 in Barbican to POSIX
+        if secret.created:
+            time_stamp = timeutils.parse_isotime(
+                str(secret.created)).timetuple()
+            created = calendar.timegm(time_stamp)
+
         if issubclass(secret_type, key_base_class.Key):
             return secret_type(secret.algorithm,
                                secret.bit_length,
                                secret_data,
-                               secret.name)
+                               secret.name,
+                               created)
         else:
             return secret_type(secret_data,
-                               secret.name)
+                               secret.name,
+                               created)
 
     def _get_secret(self, context, object_id):
         """Returns the metadata of the secret.
@@ -451,7 +500,7 @@ class BarbicanKeyManager(key_manager.KeyManager):
                 barbican_exceptions.HTTPClientError,
                 barbican_exceptions.HTTPServerError) as e:
             with excutils.save_and_reraise_exception():
-                LOG.error(u._LE("Error getting secret metadata: %s"), e)
+                LOG.error(_LE("Error getting secret metadata: %s"), e)
 
     def _is_secret_not_found_error(self, error):
         if (isinstance(error, barbican_exceptions.HTTPClientError) and
@@ -478,7 +527,7 @@ class BarbicanKeyManager(key_manager.KeyManager):
         except (barbican_exceptions.HTTPAuthError,
                 barbican_exceptions.HTTPClientError,
                 barbican_exceptions.HTTPServerError) as e:
-            LOG.error(u._LE("Error retrieving object: %s"), e)
+            LOG.error(_LE("Error retrieving object: %s"), e)
             if self._is_secret_not_found_error(e):
                 raise exception.ManagedObjectNotFoundError(
                     uuid=managed_object_id)
@@ -502,7 +551,7 @@ class BarbicanKeyManager(key_manager.KeyManager):
         except (barbican_exceptions.HTTPAuthError,
                 barbican_exceptions.HTTPClientError,
                 barbican_exceptions.HTTPServerError) as e:
-            LOG.error(u._LE("Error deleting object: %s"), e)
+            LOG.error(_LE("Error deleting object: %s"), e)
             if self._is_secret_not_found_error(e):
                 raise exception.ManagedObjectNotFoundError(
                     uuid=managed_object_id)
